import setuptools

setuptools.setup(
    name="peaqevcore",
    version="0.4.1",
    author="Magnus Eldén",
    description="Core types for peaqev car charging",
    url="https://github.com/elden1337/peaqev-core",
    license="MIT",
    packages=["peaqevcore"],
    test_requires=['pytest']
)   
